Class: Psych::ScalarScanner

Inherits:
Object show all
Defined in:
lib/psych/scalar_scanner.rb

Overview

Scan scalars for built in types

Constant Summary collapse

TIME =
/^-?\d{4}-\d{1,2}-\d{1,2}(?:[Tt]|\s+)\d{1,2}:\d\d:\d\d(?:\.\d*)?(?:\s*(?:Z|[-+]\d{1,2}:?(?:\d\d)?))?$/
FLOAT =

Taken from http://yaml.org/type/float.html Base 60, [-+]inf and NaN are handled separately

/^(?:[-+]?([0-9][0-9_,]*)?\.[0-9]*([eE][-+][0-9]+)?(?# base 10))$/x
INTEGER_STRICT =

Taken from http://yaml.org/type/int.html and modified to ensure at least one numerical symbol exists

/^(?:[-+]?0b[_]*[0-1][0-1_]*             (?# base 2)
|[-+]?0[_]*[0-7][0-7_]*              (?# base 8)
|[-+]?(0|[1-9][0-9_]*)               (?# base 10)
|[-+]?0x[_]*[0-9a-fA-F][0-9a-fA-F_]* (?# base 16))$/x
INTEGER_LEGACY =

Same as above, but allows commas. Not to YML spec, but kept for backwards compatibility

/^(?:[-+]?0b[_,]*[0-1][0-1_,]*                (?# base 2)
|[-+]?0[_,]*[0-7][0-7_,]*                 (?# base 8)
|[-+]?(?:0|[1-9](?:[0-9]|,[0-9]|_[0-9])*) (?# base 10)
|[-+]?0x[_,]*[0-9a-fA-F][0-9a-fA-F_,]*    (?# base 16))$/x
BOOLEAN_TRUE =
/^(yes|true|on)$/i
BOOLEAN_FALSE =
/^(no|false|off)$/i

Instance Attribute Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(class_loader, strict_integer: false, parse_symbols: true) ⇒ ScalarScanner

Create a new scanner



42
43
44
45
46
47
# File 'lib/psych/scalar_scanner.rb', line 42

def initialize class_loader, strict_integer: false, parse_symbols: true
  @symbol_cache = {}
  @class_loader = class_loader
  @strict_integer = strict_integer
  @parse_symbols = parse_symbols
end

Instance Attribute Details

#class_loaderObject (readonly)

Returns the value of attribute class_loader.



39
40
41
# File 'lib/psych/scalar_scanner.rb', line 39

def class_loader
  @class_loader
end

Instance Method Details

#parse_int(string) ⇒ Object

Parse and return an int from string



121
122
123
# File 'lib/psych/scalar_scanner.rb', line 121

def parse_int string
  Integer(string.delete(',_'))
end

#parse_time(string) ⇒ Object

Parse and return a Time from string



127
128
129
130
131
132
133
134
135
136
137
138
139
140
141
142
143
144
145
146
147
148
149
150
151
152
# File 'lib/psych/scalar_scanner.rb', line 127

def parse_time string
  klass = class_loader.load 'Time'

  date, time = *(string.split(/[Tt]|\s+/, 2))
  (yy, m, dd) = date.match(/^(-?\d{4})-(\d{1,2})-(\d{1,2})/).captures.map { |x| x.to_i }
  md = time.match(/(\d+:\d+:\d+)(?:\.(\d*))?\s*(Z|[-+]\d+(:\d\d)?)?/)

  (hh, mm, ss) = md[1].split(':').map { |x| x.to_i }
  us = (md[2] ? Rational("0.#{md[2]}") : 0) * 1000000

  time = klass.utc(yy, m, dd, hh, mm, ss, us)

  return time if 'Z' == md[3]
  return klass.at(time.to_i, us) unless md[3]

  tz = md[3].match(/^([+\-]?\d{1,2})\:?(\d{1,2})?$/)[1..-1].compact.map { |digit| Integer(digit, 10) }
  offset = tz.first * 3600

  if offset < 0
    offset -= ((tz[1] || 0) * 60)
  else
    offset += ((tz[1] || 0) * 60)
  end

  klass.new(yy, m, dd, hh, mm, ss+us/(1_000_000r), offset)
end

#tokenize(string) ⇒ Object

Tokenize string returning the Ruby object



50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
81
82
83
84
85
86
87
88
89
90
91
92
93
94
95
96
97
98
99
100
101
102
103
104
105
106
107
108
109
110
111
112
113
114
115
116
117
# File 'lib/psych/scalar_scanner.rb', line 50

def tokenize string
  return nil if string.empty?
  return @symbol_cache[string] if @symbol_cache.key?(string)
  integer_regex = @strict_integer ? INTEGER_STRICT : INTEGER_LEGACY
  # Check for a String type, being careful not to get caught by hash keys, hex values, and
  # special floats (e.g., -.inf).
  if string.match?(%r{^[^\d.:-]?[[:alpha:]_\s!@#$%\^&*(){}<>|/\\~;=]+}) || string.match?(/\n/)
    return string if string.length > 5

    if string.match?(/^[^ytonf~]/i)
      string
    elsif string == '~' || string.match?(/^null$/i)
      nil
    elsif string.match?(BOOLEAN_TRUE)
      true
    elsif string.match?(BOOLEAN_FALSE)
      false
    else
      string
    end
  elsif string.match?(TIME)
    begin
      parse_time string
    rescue ArgumentError
      string
    end
  elsif string.match?(/^\d{4}-(?:1[012]|0\d|\d)-(?:[12]\d|3[01]|0\d|\d)$/)
    begin
      class_loader.date.strptime(string, '%F', Date::GREGORIAN)
    rescue ArgumentError
      string
    end
  elsif string.match?(/^\+?\.inf$/i)
    Float::INFINITY
  elsif string.match?(/^-\.inf$/i)
    -Float::INFINITY
  elsif string.match?(/^\.nan$/i)
    Float::NAN
  elsif @parse_symbols && string.match?(/^:./)
    if string =~ /^:(["'])(.*)\1/
      @symbol_cache[string] = class_loader.symbolize($2.sub(/^:/, ''))
    else
      @symbol_cache[string] = class_loader.symbolize(string.sub(/^:/, ''))
    end
  elsif string.match?(/^[-+]?[0-9][0-9_]*(:[0-5]?[0-9]){1,2}$/)
    i = 0
    string.split(':').each_with_index do |n,e|
      i += (n.to_i * 60 ** (e - 2).abs)
    end
    i
  elsif string.match?(/^[-+]?[0-9][0-9_]*(:[0-5]?[0-9]){1,2}\.[0-9_]*$/)
    i = 0
    string.split(':').each_with_index do |n,e|
      i += (n.to_f * 60 ** (e - 2).abs)
    end
    i
  elsif string.match?(FLOAT)
    if string.match?(/\A[-+]?\.\Z/)
      string
    else
      Float(string.delete(',_').gsub(/\.([Ee]|$)/, '\1'))
    end
  elsif string.match?(integer_regex)
    parse_int string
  else
    string
  end
end